"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @dgProcessor
 *
 * @description
 * Merge the description from `@param` tags into the parameter docs
 * extracted from the TypeScript
 */
function mergeParameterInfo() {
    return new MergeParameterInfoProcessor();
}
exports.mergeParameterInfo = mergeParameterInfo;
var MergeParameterInfoProcessor = /** @class */ (function () {
    function MergeParameterInfoProcessor() {
        this.$runAfter = ['readTypeScriptModules', 'tags-extracted'];
        this.$runBefore = ['extra-docs-added'];
    }
    MergeParameterInfoProcessor.prototype.$process = function (docs) {
        docs.forEach(function (doc) {
            if (doc.docType === 'parameter') {
                // The `params` property comes from parsing the `@param` jsdoc tags on the container doc
                var paramTag = doc.container.params && doc.container.params.find(function (param) { return param.name === doc.name; });
                if (paramTag && paramTag.description) {
                    doc.description = paramTag.description;
                }
            }
        });
    };
    return MergeParameterInfoProcessor;
}());
exports.MergeParameterInfoProcessor = MergeParameterInfoProcessor;
//# sourceMappingURL=mergeParameterInfo.js.map